import unittest

from osrf_pycommon.terminal_color import impl

assert impl._enabled is True


class TestTerminalColorImpl(unittest.TestCase):
    test_format_str = "@{r}red @!bold red @|normal @{rb}red bg"
    test_str = "\x1b[31mred \033[1mbold red \x1b[0mnormal \x1b[41mred bg"

    def test_ansi(self):
        ansi = impl.ansi
        self.assertEqual('\x1b[0m', ansi('reset'))
        impl.disable_ansi_color_substitution_globally()
        self.assertNotEqual('\x1b[0m', ansi('reset'))
        self.assertEqual('@|', ansi('atbar'))
        impl.enable_ansi_color_substitution_globally()
        self.assertEqual('\x1b[0m', ansi('reset'))

    def test_get_ansi_dict(self):
        get_ansi_dict = impl.get_ansi_dict
        ansi_dict = get_ansi_dict()
        self.assertNotEqual({}, ansi_dict)
        self.assertEqual('\x1b[0m', ansi_dict['reset'])
        impl.disable_ansi_color_substitution_globally()
        ansi_dict = get_ansi_dict()
        self.assertEqual('\x1b[0m', ansi_dict['reset'])
        self.assertEqual('@|', ansi_dict['atbar'])
        self.assertNotEqual({}, ansi_dict)
        impl.enable_ansi_color_substitution_globally()
        ansi_dict = get_ansi_dict()
        self.assertNotEqual({}, ansi_dict)
        self.assertEqual('\x1b[0m', ansi_dict['reset'])

    def test_enable_and_disable_ansi_color_substitution_globally(self):
        enable = impl.enable_ansi_color_substitution_globally
        disable = impl.disable_ansi_color_substitution_globally
        is_windows = impl._is_windows
        enabled = impl._enabled
        try:
            impl._is_windows = False
            impl._enabled = True
            enable()
            self.assertEqual('\x1b[0m', impl.ansi('reset'))
            self.assertEqual('\x1b[0m', impl.format_color('@|'))
            disable()
            self.assertEqual('', impl.ansi('reset'))
            self.assertEqual('', impl.format_color('@|'))
            enable()
            self.assertEqual('\x1b[0m', impl.ansi('reset'))
            self.assertEqual('\x1b[0m', impl.format_color('@|'))
        finally:
            impl._is_windows = is_windows
            impl._enabled = enabled

    def test_format_color(self):
        is_windows = impl._is_windows
        try:
            impl._is_windows = False
            format_color = impl.format_color
            self.assertEqual(self.test_str, format_color(self.test_format_str))
            sanitized_str = "|@@ Notice @{atbar}"
            self.assertEqual("|@ Notice @|", format_color(sanitized_str))
        finally:
            impl._is_windows = is_windows

    def test_sanitize(self):
        sanitize = impl.sanitize
        test_str = "Email: {email}@{org}"
        self.assertEqual("Email: {{email}}@@{{org}}", sanitize(test_str))
        test_str = "|@ Notice @|"
        self.assertEqual("|@@ Notice @{atbar}", sanitize(test_str))



import sys
import unittest

from osrf_pycommon.cli_utils import verb_pattern

called = None


class TestCliUtilsVerbPattern(unittest.TestCase):
    def test_call_prepare_arguments(self):
        global called
        cpa = verb_pattern.call_prepare_arguments

        # Try with basic, one parameter
        called = False

        def fake_prepare_arguments(parser):
            global called
            called = True
            if called:
                pass
            return parser

        r = cpa(fake_prepare_arguments, None)
        self.assertTrue(called)
        self.assertIsNone(r)

        # Try with args
        called = False

        def fake_prepare_arguments(parser, args):
            global called
            called = True
            if called:
                pass
            return parser

        r = cpa(fake_prepare_arguments, None)
        self.assertTrue(called)
        self.assertIsNone(r)

        # Try with self
        called = False

        class Foo:
            def fake_prepare_arguments(self, parser, args):
                global called
                called = True
                if called:
                    pass
                return parser

        f = Foo()
        r = cpa(f.fake_prepare_arguments, None)
        self.assertTrue(called)
        self.assertIsNone(r)

        # Try with more than needed
        called = False

        class Foo:
            def fake_prepare_arguments(self, parser, args, extra):
                global called
                called = True
                if called:
                    pass
                return parser

        f = Foo()

        # Remove the following if condition and keep else condition once
        # Xenial is dropped
        if sys.version_info[0] < 3:
            with self.assertRaisesRegexp(ValueError, 'one or two parameters'):
                r = cpa(f.fake_prepare_arguments, None)

        else:
            with self.assertRaisesRegex(ValueError, 'one or two parameters'):
                r = cpa(f.fake_prepare_arguments, None)

        # Try with less than needed
        called = False

        class Foo:
            def fake_prepare_arguments(self):
                global called
                called = True
                if called:
                    pass
                return 'Should not get here'

        f = Foo()

        # Remove the following if condition and keep else condition once
        # Xenial is dropped
        if sys.version_info[0] < 3:
            with self.assertRaisesRegexp(ValueError, 'one or two parameters'):
                r = cpa(f.fake_prepare_arguments, None)

        else:
            with self.assertRaisesRegex(ValueError, 'one or two parameters'):
                r = cpa(f.fake_prepare_arguments, None)

        # Try with additional optional argument
        called = False

        class Foo:
            def fake_prepare_arguments(self, parser, args, optional=None):
                global called
                called = True
                if called:
                    pass
                return parser

        f = Foo()
        r = cpa(f.fake_prepare_arguments, None)
        self.assertTrue(called)
        self.assertIsNone(r)

    def test_split_arguments_by_verb(self):
        args = ['--cmd-arg1', 'verb', '--verb-arg1', '--verb-arg2']
        expected = ('verb', ['--cmd-arg1'], ['--verb-arg1', '--verb-arg2'])
        self.assertEqual(verb_pattern.split_arguments_by_verb(args), expected)

        args = ['verb', '--verb-arg1', '--verb-arg2']
        expected = ('verb', [], ['--verb-arg1', '--verb-arg2'])
        self.assertEqual(verb_pattern.split_arguments_by_verb(args), expected)

        args = ['--cmd-arg1', 'verb']
        expected = ('verb', ['--cmd-arg1'], [])
        self.assertEqual(verb_pattern.split_arguments_by_verb(args), expected)

        args = ['verb']
        expected = ('verb', [], [])
        self.assertEqual(verb_pattern.split_arguments_by_verb(args), expected)

        args = ['--cmd-arg1']
        expected = (None, ['--cmd-arg1'], [])
        self.assertEqual(verb_pattern.split_arguments_by_verb(args), expected)



import os
import sys
import unittest

this_dir = os.path.dirname(os.path.abspath(__file__))

test_script = os.path.join(
    this_dir,
    'fixtures',
    'execute_process',
    'stdout_stderr_ordering.py')
python = sys.executable


def convert_file_linesep_with_pty_linesep(string):
    return string.replace(b"""
""", b"\r\n")


@unittest.skipIf(sys.platform.startswith("win"), "Windows not supported")
class TestProcessUtilsExecuteNoPty(unittest.TestCase):
    def test__execute_process_pty_combined_unbuffered(self):
        from osrf_pycommon.process_utils import execute_process_pty
        exc_pty = execute_process_pty._execute_process_pty

        # Test ordering with stdout and stderr combined and Python unbuffered
        cmd = [python, "-u", test_script]
        result = b""
        for out, err, ret in exc_pty(cmd, None, None, False, True):
            if out is not None:
                result += out
            if err is not None:
                result += err
            if ret is not None:
                break
        expected = convert_file_linesep_with_pty_linesep(b"""\
out 1
err 1
out 2
""")
        self.assertEqual(expected, result)

    def test__execute_process_pty_unbuffered(self):
        from osrf_pycommon.process_utils import execute_process_pty
        exc_pty = execute_process_pty._execute_process_pty

        # Test ordering with stdout and stderr combined and Python unbuffered
        cmd = [python, "-u", test_script]
        result = b""
        for out, err, ret in exc_pty(cmd, None, None, False, False):
            if out is not None:
                result += out
            if err is not None:
                result += err
            if ret is not None:
                break
        expected = convert_file_linesep_with_pty_linesep(b"""\
out 1
err 1
out 2
""")
        self.assertEqual(expected, result)

    def test__execute_process_pty_combined(self):
        from osrf_pycommon.process_utils import execute_process_pty
        exc_pty = execute_process_pty._execute_process_pty

        # Test ordering with stdout and stderr combined
        cmd = [python, test_script]
        result = b""
        for out, err, ret in exc_pty(cmd, None, None, False, True):
            if out is not None:
                result += out
            if err is not None:
                result += err
            if ret is not None:
                break
        expected = convert_file_linesep_with_pty_linesep(b"""\
out 1
err 1
out 2
""")
        self.assertEqual(expected, result)

    def test__execute_process_pty(self):
        from osrf_pycommon.process_utils import execute_process_pty
        exc_pty = execute_process_pty._execute_process_pty

        # Test ordering with stdout and stderr separate
        cmd = [python, test_script]
        result = b""
        for out, err, ret in exc_pty(cmd, None, None, False, False):
            if out is not None:
                result += out
            if err is not None:
                result += err
            if ret is not None:
                break
        expected = convert_file_linesep_with_pty_linesep(b"""\
out 1
err 1
out 2
""")
        self.assertEqual(expected, result)



from osrf_pycommon.process_utils import AsyncSubprocessProtocol


def create_protocol():
    class CustomProtocol(AsyncSubprocessProtocol):
        def __init__(self, **kwargs):
            self.stdout_buffer = b""
            self.stderr_buffer = b""
            AsyncSubprocessProtocol.__init__(self, **kwargs)

        def on_stdout_received(self, data):
            self.stdout_buffer += data

        def on_stderr_received(self, data):
            self.stderr_buffer += data
    return CustomProtocol



import unittest
from unittest import mock

from osrf_pycommon.terminal_utils import is_tty


class TestTerminalUtils(unittest.TestCase):
    def test_is_tty(self):
        mock_stream = object()
        self.assertFalse(is_tty(mock_stream))
        mock_stream = mock.MagicMock()
        mock_stream.isatty = mock.MagicMock(return_value=None)
        self.assertFalse(is_tty(mock_stream))
        mock_stream.isatty = mock.MagicMock(return_value=False)
        self.assertFalse(is_tty(mock_stream))
        mock_stream.isatty = mock.MagicMock(return_value=True)
        self.assertTrue(is_tty(mock_stream))



from osrf_pycommon.cli_utils import common


def test_extract_jobs_flags():
    extract_jobs_flags = common.extract_jobs_flags
    valid_mflags = [
        '-j8 -l8', 'j8 ', '-j', 'j', '-l8', 'l8',
        '-l', 'l', '-j18', ' -j8 l9', '-j1 -l1',
        '--jobs=8', '--jobs 8', '--jobs', '--load-average',
        '--load-average=8', '--load-average 8', '--jobs=8 -l9'
    ]
    results = [
        '-j8 -l8', 'j8', '-j', 'j', '-l8', 'l8',
        '-l', 'l', '-j18', '-j8 l9', '-j1 -l1',
        '--jobs=8', '--jobs 8', '--jobs', '--load-average',
        '--load-average=8', '--load-average 8', '--jobs=8 -l9'
    ]
    leftover_results = [
        '', ' ', '', '', '', '',
        '', '', '', '', '',
        '', '', '', '',
        '', '', ''
    ]
    for mflag, result in zip(valid_mflags, zip(results, leftover_results)):
        result, leftover_result = result
        matches, leftovers = extract_jobs_flags(mflag)
        print('--')
        print("input:    '{0}'".format(mflag))
        print("matched:  '{0}'".format(matches))
        print("expected: '{0}'".format(result))
        assert matches == result, "should match '{0}'".format(mflag)
        assert leftovers == leftover_result, "expected leftovers '{0}': '{1}'"\
            .format(leftovers)
    invalid_mflags = ['', '--jobs= 8', '--jobs8']
    for mflag in invalid_mflags:
        matches, leftovers = extract_jobs_flags(mflag)
        assert matches == '', "should not match '{0}'".format(mflag)
        assert leftovers == mflag, "'{0}' should be in leftovers: '{1}'"\
            .format(mflag, leftovers)
    mixed_flags = [
        'target -j8 -l8 --other-option'
    ]
    results = [
        ('-j8 -l8', 'target --other-option')
    ]
    for args, result in zip(mixed_flags, results):
        expected_matches, expected_leftovers = result
        matches, leftovers = extract_jobs_flags(args)
        assert matches == expected_matches, "should have matched '{0}'"\
            .format(expected_matches)
        assert leftovers == expected_leftovers, "should have left '{0}' '{1}'"\
            .format(expected_leftovers, leftovers)


def test_extract_argument_group():
    extract_argument_group = common.extract_argument_group
    # Example 1 from docstring
    args = ['foo', '--args', 'bar', '--baz']
    expected = (['foo'], ['bar', '--baz'])
    results = extract_argument_group(args, '--args')
    assert expected == results, (args, expected, results)
    # Example 2 from docstring
    args = ['foo', '--args', 'bar', '--baz', '---', '--', '--foo-option']
    expected = (['foo', '--foo-option'], ['bar', '--baz', '--'])
    results = extract_argument_group(args, '--args')
    assert expected == results, (args, expected, results)
    # Example 3 from docstring
    args = ['foo',
            '--args', 'ping', '--',
            'bar',
            '--args', 'pong', '--',
            'baz',
            '--args', '--']
    expected = (['foo', 'bar', 'baz'], ['ping', 'pong'])
    results = extract_argument_group(args, '--args')
    assert expected == results, (args, expected, results)
    # Example with delimiter but no arguments
    args = ['foo', '--args']
    expected = (['foo'], [])
    results = extract_argument_group(args, '--args')
    assert expected == results, (args, expected, results)
    # Example with no delimiter
    args = ['foo', 'bar']
    expected = (['foo', 'bar'], [])
    results = extract_argument_group(args, '--args')
    assert expected == results, (args, expected, results)



import os
import sys
import unittest

from osrf_pycommon.process_utils import execute_process_nopty

this_dir = os.path.dirname(os.path.abspath(__file__))

test_script = os.path.join(
    this_dir,
    'fixtures',
    'execute_process',
    'stdout_stderr_ordering.py')
python = sys.executable

file_nl = b"""
"""
nl = os.linesep.encode()


class TestProcessUtilsExecuteNoPty(unittest.TestCase):
    def test__process_incomming_lines(self):
        pil = execute_process_nopty._process_incoming_lines

        # Test with no left overs and no new incoming
        left_overs = b''
        incoming = b''
        self.assertEqual((None, left_overs), pil(incoming, left_overs))

        # Test with left overs, but no new incoming
        left_overs = b'something'
        incoming = b''
        self.assertEqual((b'', left_overs), pil(incoming, left_overs))

        # Test with no left overs, but new incoming
        left_overs = b''
        incoming = nl.join([b'one', b'two'])
        self.assertEqual((b'one' + nl, b'two'), pil(incoming, left_overs))

        # Test with left overs and new incoming with prefixed nl
        left_overs = b'something'
        incoming = (nl + b'else')
        expected = (b'something' + nl, b'else')
        self.assertEqual(expected, pil(incoming, left_overs))

    def test__execute_process_nopty_combined_unbuffered(self):
        exc_nopty = execute_process_nopty._execute_process_nopty

        # Test ordering with stdout and stderr combined and Python unbuffered
        cmd = [python, "-u", test_script]
        result = b""
        for out, err, ret in exc_nopty(cmd, None, None, False, True):
            if out is not None:
                result += out
            if err is not None:
                result += err
            if ret is not None:
                break
        expected = b"""\
out 1
err 1
out 2
"""
        expected = expected.replace(file_nl, nl)
        expected = sorted(expected.splitlines(True))
        result = sorted(result.splitlines(True))
        self.assertEqual(expected, result)

    def test__execute_process_nopty_unbuffered(self):
        exc_nopty = execute_process_nopty._execute_process_nopty

        # Test ordering with stdout and stderr combined and Python unbuffered
        cmd = [python, "-u", test_script]
        result = b""
        for out, err, ret in exc_nopty(cmd, None, None, False, False):
            if out is not None:
                result += out
            if err is not None:
                result += err
            if ret is not None:
                break
        expected = b"""\
out 1
err 1
out 2
"""
        expected = expected.replace(file_nl, nl)
        expected = sorted(expected.splitlines(True))
        result = sorted(result.splitlines(True))
        self.assertEqual(expected, result)

    def test__execute_process_nopty_combined(self):
        exc_nopty = execute_process_nopty._execute_process_nopty

        # Test ordering with stdout and stderr combined
        cmd = [python, test_script]
        result = b""
        for out, err, ret in exc_nopty(cmd, None, None, False, True):
            if out is not None:
                result += out
            if err is not None:
                result += err
            if ret is not None:
                break
        expected = b"""\
out 1
err 1
out 2
"""
        expected = expected.replace(file_nl, nl)
        expected = sorted(expected.splitlines(True))
        result = sorted(result.splitlines(True))
        self.assertEqual(expected, result)

    def test__execute_process_nopty(self):
        exc_nopty = execute_process_nopty._execute_process_nopty

        # Test ordering with stdout and stderr separate
        cmd = [python, test_script]
        result = b""
        for out, err, ret in exc_nopty(cmd, None, None, False, False):
            if out is not None:
                result += out
            if err is not None:
                result += err
            if ret is not None:
                break
        expected = b"""\
out 1
err 1
out 2
"""
        expected = expected.replace(file_nl, nl)
        expected = sorted(expected.splitlines(True))
        result = sorted(result.splitlines(True))
        self.assertEqual(expected, result)



import unittest

from osrf_pycommon.terminal_color import ansi_re


class TestTerminalColorAnsiRe(unittest.TestCase):
    test_str = "\x1b[31mred \033[1mbold red \x1b[0mnormal \x1b[41mred bg"

    def test_split_by_ansi_escape_sequence(self):
        split_ansi = ansi_re.split_by_ansi_escape_sequence
        expected = [
            "", "\x1b[31m", "red ", "\x1b[1m", "bold red ", "\x1b[0m",
            "normal ", "\x1b[41m", "red bg"
        ]
        self.assertEqual(expected, split_ansi(self.test_str, True))
        expected = ["", "red ", "bold red ", "normal ", "red bg"]
        self.assertEqual(expected, split_ansi(self.test_str, False))

    def test_remove_ansi_escape_sequences(self):
        remove_ansi = ansi_re.remove_ansi_escape_sequences
        expected = "red bold red normal red bg"
        self.assertEqual(expected, remove_ansi(self.test_str))

    def test_remove_ansi_escape_sequences_false_positives(self):
        remove_ansi = ansi_re.remove_ansi_escape_sequences
        false_positive = "Should not match: \1xb[1234m \033[m \1xb[3Om"
        self.assertEqual(false_positive, remove_ansi(false_positive))



from __future__ import unicode_literals

import os
import unittest

from osrf_pycommon.process_utils import impl

this_dir = os.path.dirname(os.path.abspath(__file__))


class TestProcessUtilsImpl(unittest.TestCase):
    def test_which(self):
        which = impl.which
        j = os.path.join
        prefix = j(this_dir, 'fixtures', 'impl_which')
        paths = os.pathsep.join([
            j(prefix, 'usr', 'local', 'bin'),
            j(prefix, 'usr', 'bin'),
            j(prefix, 'bin'),
        ])
        # bin_only exists +x in bin only
        expected = j(prefix, 'bin', 'bin_only.exe')
        self.assertEqual(expected, which('bin_only.exe', path=paths))
        self.assertEqual(expected, which(expected, path=paths))
        # exc1 exists +x in bin and usr/bin
        expected = j(prefix, 'usr', 'bin', 'exc1.exe')
        self.assertEqual(expected, which('exc1.exe', path=paths))
        self.assertEqual(expected, which(expected, path=paths))
        # exc2 exists +x in bin and usr/bin, but -x in usr/local/bin
        expected = j(prefix, 'usr', 'bin', 'exc2.exe')
        self.assertEqual(expected, which('exc2.exe', path=paths))
        self.assertEqual(expected, which(expected, path=paths))
        # Same as above, with PATH
        orig_path = os.environ['PATH']
        try:
            os.environ['PATH'] = paths
            # bin_only exists +x in bin only
            expected = j(prefix, 'bin', 'bin_only.exe')
            self.assertEqual(expected, which('bin_only.exe'))
            self.assertEqual(expected, which(expected))
            # exc1 exists +x in bin and usr/bin
            expected = j(prefix, 'usr', 'bin', 'exc1.exe')
            self.assertEqual(expected, which('exc1.exe'))
            self.assertEqual(expected, which(expected))
            # exc2 exists +x in bin and usr/bin, but -x in usr/local/bin
            expected = j(prefix, 'usr', 'bin', 'exc2.exe')
            self.assertEqual(expected, which('exc2.exe'))
            self.assertEqual(expected, which(expected))
        finally:
            os.environ['PATH'] = orig_path
        which(str("exc1.exe"), path=paths)  # Make sure unicode/str works



import atexit
import os
import sys
import unittest

from .impl_aep_asyncio import run
from .impl_aep_asyncio import loop

this_dir = os.path.dirname(os.path.abspath(__file__))

test_script = os.path.join(
    this_dir,
    'fixtures',
    'execute_process',
    'stdout_stderr_ordering.py')
test_script_quoted = '"%s"' % test_script if ' ' in test_script else test_script
python = sys.executable


# This atexit handler ensures the loop is closed after all tests were run.
@atexit.register
def close_loop():
    if not loop.is_closed():
        loop.close()


class TestProcessUtilsAsyncExecuteProcess(unittest.TestCase):
    def test_async_execute_process_no_emulation_shell_false_combined(self):
        stdout, stderr, retcode = loop.run_until_complete(run(
            [python, test_script]))
        stdout, stderr = stdout.decode(), stderr.decode()
        self.assertIn('out 1', stdout)
        self.assertIn('err 1', stdout)
        self.assertIn('out 2', stdout)
        self.assertEqual('', stderr)
        self.assertEqual(0, retcode)

    def test_async_execute_process_no_emulation_shell_true_combined(self):
        stdout, stderr, retcode = loop.run_until_complete(run(
            [python, test_script_quoted], shell=True))
        stdout, stderr = stdout.decode(), stderr.decode()
        self.assertIn('out 1', stdout)
        self.assertIn('err 1', stdout)
        self.assertIn('out 2', stdout)
        self.assertEqual('', stderr)
        self.assertEqual(0, retcode)

    def test_async_execute_process_no_emulation_shell_false(self):
        stdout, stderr, retcode = loop.run_until_complete(run(
            [python, test_script], stderr_to_stdout=False))
        stdout, stderr = stdout.decode(), stderr.decode()
        self.assertIn('out 1', stdout)
        self.assertIn('err 1', stderr)
        self.assertIn('out 2', stdout)
        self.assertEqual(0, retcode)

    def test_async_execute_process_no_emulation_shell_true(self):
        stdout, stderr, retcode = loop.run_until_complete(run(
            [python, test_script_quoted], stderr_to_stdout=False, shell=True))
        stdout, stderr = stdout.decode(), stderr.decode()
        self.assertIn('out 1', stdout)
        self.assertIn('err 1', stderr)
        self.assertIn('out 2', stdout)
        self.assertEqual(0, retcode)

    def test_async_execute_process_with_emulation_shell_false_combined(self):
        stdout, stderr, retcode = loop.run_until_complete(run(
            [python, test_script], emulate_tty=True))
        stdout, stderr = stdout.decode(), stderr.decode()
        self.assertIn('out 1', stdout)
        self.assertIn('err 1', stdout)
        self.assertIn('out 2', stdout)
        self.assertEqual('', stderr)
        self.assertEqual(0, retcode)

    def test_async_execute_process_with_emulation_shell_true_combined(self):
        stdout, stderr, retcode = loop.run_until_complete(run(
            [python, test_script_quoted], emulate_tty=True, shell=True))
        stdout, stderr = stdout.decode(), stderr.decode()
        self.assertIn('out 1', stdout)
        self.assertIn('err 1', stdout)
        self.assertIn('out 2', stdout)
        self.assertEqual('', stderr)
        self.assertEqual(0, retcode)

    def test_async_execute_process_with_emulation_shell_false(self):
        stdout, stderr, retcode = loop.run_until_complete(run(
            [python, test_script], emulate_tty=True, stderr_to_stdout=False))
        stdout, stderr = stdout.decode(), stderr.decode()
        self.assertIn('out 1', stdout)
        self.assertIn('err 1', stderr)
        self.assertIn('out 2', stdout)
        self.assertEqual(0, retcode)

    def test_async_execute_process_with_emulation_shell_true(self):
        stdout, stderr, retcode = loop.run_until_complete(run(
            [python, test_script_quoted], emulate_tty=True, stderr_to_stdout=False,
            shell=True))
        stdout, stderr = stdout.decode(), stderr.decode()
        self.assertIn('out 1', stdout)
        self.assertIn('err 1', stderr)
        self.assertIn('out 2', stdout)
        self.assertEqual(0, retcode)



import os
import subprocess
import sys


def test_flake8():
    """Test source code for pyFlakes and PEP8 conformance"""
    this_dir = os.path.dirname(os.path.abspath(__file__))
    source_dir = os.path.join(this_dir, '..', 'osrf_pycommon')
    cmd = [sys.executable, '-m', 'flake8', source_dir, '--count']
    # if flake8_import_order is installed, set the style to google
    try:
        import flake8_import_order  # noqa
        cmd.extend(['--import-order-style=google'])
    except ImportError:
        pass
    # ignore error codes from plugins this package doesn't comply with
    cmd.extend(['--ignore=C,D,Q,I'])
    # work around for https://gitlab.com/pycqa/flake8/issues/179
    cmd.extend(['--jobs', '1'])
    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    stdout, stderr = p.communicate()
    print(stdout)
    assert p.returncode == 0, \
        "Command '{0}' returned non-zero exit code '{1}'".format(' '.join(cmd), p.returncode)



from osrf_pycommon.process_utils.async_execute_process import async_execute_process
from osrf_pycommon.process_utils import get_loop

from .impl_aep_protocol import create_protocol

loop = get_loop()


async def run(cmd, **kwargs):
    transport, protocol = await async_execute_process(
        create_protocol(), cmd, **kwargs)
    retcode = await protocol.complete
    transport.close()
    return protocol.stdout_buffer, protocol.stderr_buffer, retcode



# Copyright 2014 Open Source Robotics Foundation, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import threading
import warnings

_thread_local = threading.local()


def get_loop_impl(asyncio):
    # See the note in
    # https://docs.python.org/3/library/asyncio-eventloop.html#asyncio.get_event_loop # noqa
    # In short, between Python 3.10.0 and 3.10.8, this unconditionally raises a
    # DeprecationWarning.  But after 3.10.8, it only raises the warning if
    # ther eis no current loop set in the policy.  Since we are setting a loop
    # in the policy, this warning is spurious, and will go away once we get
    # away from Python 3.10.6 (verified with Python 3.11.3).
    with warnings.catch_warnings():
        warnings.filterwarnings(
            'ignore',
            'There is no current event loop',
            DeprecationWarning)

        global _thread_local
        if getattr(_thread_local, 'loop_has_been_setup', False):
            return asyncio.get_event_loop()
        # Setup this thread's loop and return it
        if os.name == 'nt':
            try:
                loop = asyncio.get_event_loop()
                if not isinstance(loop, asyncio.ProactorEventLoop):
                    # Before replacing the existing loop, explicitly
                    # close it to prevent an implicit close during
                    # garbage collection, which may or may not be a
                    # problem depending on the loop implementation.
                    loop.close()
                    loop = asyncio.ProactorEventLoop()
                    asyncio.set_event_loop(loop)
            except (RuntimeError, AssertionError):
                loop = asyncio.ProactorEventLoop()
                asyncio.set_event_loop(loop)
        else:
            try:
                loop = asyncio.get_event_loop()
            except (RuntimeError, AssertionError):
                loop = asyncio.new_event_loop()
                asyncio.set_event_loop(loop)
        _thread_local.loop_has_been_setup = True

    return loop
